/*
 * Resource.java
 * 
 * Created on 9-sep-2007, 15:10:28
 * 
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package net.sf.sql2java.common.resources;

import java.io.File;

import net.sf.sql2java.common.parameters.StringParser;
import net.sf.sql2java.common.parameters.handlers.SimpleParameterHandler;

/**
 *
 * @author dge2
 */
public class Resource {
    public static final String DELIMITER = ":{delimiter}";
    
    private StringBuffer content;
    private StringBuffer raw;
    private SimpleParameterHandler handler;
    private StringParser parser;
    
// -- Constructors ------------------------------------------------------------
    
    public Resource(String init) {
    	this();
    
    	// -- add the initial value to the content
    	content.append(init);
    }
    
    public Resource() {
    	handler = new SimpleParameterHandler();
    	parser = new StringParser(handler);
    	
    	content = new StringBuffer();
    	raw = new StringBuffer();
    }
    
// -- Resource Manipulation ---------------------------------------------------
    
    /**
     * Add a part to the resource.
     * 
     * Later on, we'll parse the content to replace it with parameters.
     * 
     * @param part The part we want to add
     */
    public void append(String part) {
    	// -- return if part was null
    	if (part == null) return;
    	
        // -- replace all '/' and '.' with the delimiter char
        part = part.replaceAll("/", DELIMITER);
        part = part.replaceAll("\\\\", DELIMITER);
        //part = part.replaceAll("\\.", DELIMITER);
        
        // -- check if the part ends with a delimiter
        if ((content.length() > 0) && (! content.toString().endsWith(DELIMITER)))
            this.content.append(DELIMITER);
        
        // -- add the part to the cache
        content.append(part);
    }
    
    /**
     * Add a part to the resource.
     * 
     * Later on, we'll parse the content to replace it with parameters.
     * 
     * @param part The part we want to add
     */
    public void prepend(String part) {
    	// -- return if part was null
    	if (part == null) return;
    	
        // -- replace all '/' and '.' with the delimiter char
        part = part.replaceAll("/", DELIMITER);
        part = part.replaceAll("\\\\", DELIMITER);
        //part = part.replaceAll("\\.", DELIMITER);
        
        // -- check if the part ends with a delimiter
        if ((content.length() > 0) && (! content.toString().endsWith(DELIMITER)))
            this.content.insert(0, DELIMITER);
        
        // -- add the part to the cache
        content.insert(0, part);
    }
    
    /**
     * Add a raw part to the Resource content.
     * 
     * This part will not be parsed or searched for parameters.
     * 
     * @param part The part we want to add.
     */
    public void addRaw(String part) {
        // -- add the part to the cache
        raw.append(part);
    }
    
    /**
     * Add a parameter.
     * 
     * Add the value for the given parameter key. This key will be searched
     * inside the content of the resource and will be replaced by it's value
     * 
     * @param key	The parameter name
     * @param value	The parameter value
     */
    public void addParameter(String key, String value) {
    	handler.addReplacement(key, value);
    }
    
    /**
     * Clear the content of the resource
     */
    public void clear() {
        this.content = new StringBuffer();
        this.raw = new StringBuffer();
    }
    
// -- Parsing -----------------------------------------------------------------
    
    public void parse(StringParser parser) {
    	content = parser.parse(content);
    	raw = parser.parse(raw);
    }
    
// -- Getters -----------------------------------------------------------------
    
    /**
     * Get the resource as a package.
     * 
     * The content will be converted to a package String. This means all part
     * separators will be replaced by dots (.). Afterwards we'll parse the 
     * content to replace the parameter names with their values. Last thing
     * to do is adding the raw part.
     * 
     * @return the content of this resource, formed as a package
     */
    public String getPackage() {
        // -- create the result object
        String result = "";
        
        // -- add the delimiter
        handler.addReplacement("delimiter", "\\.");
        
        // -- parse the content
        result = parser.parse(content.toString());
        
        // -- append raw data
        result += raw;
        
        // -- return the result
        return result;
    }
    
    /**
     * Get the resource as a path.
     * 
     * The content will be converted to a path String. This means all part
     * separators will be replaced by the systems file separator. Afterwards 
     * we'll parse the content to replace the parameter names with their 
     * values. Last thing to do is adding the raw part.
     * 
     * @return the content of this resource, formed as a path
     */
    public String getPath() {
    	// -- create the result object
        String result = "";
        
        // -- add the delimiter
        handler.addReplacement("delimiter", "\\" + File.separator);
        
        // -- parse the content
        result = parser.parse(content.toString());
        
        // -- append raw data
        result += raw;
        
        // -- return the result
        return result;
    }
}
